@@ -62,6 +62,10 @@ class FetchFailedException(IOError):
     """Exception that is thrown when fetching fails."""
 
 
+class UntrustedException(FetchFailedException):
+    """Exception that is thrown when fetching fails for trust reasons"""
+
+
 class LockFailedException(IOError):
     """Exception that is thrown when locking fails."""
 
@@ -70,6 +74,29 @@ class CacheClosedException(Exception):
     """Exception that is thrown when the cache is used after close()."""
 
 
+class _WrappedLock(object):
+    """Wraps an apt_pkg.FileLock to raise LockFailedException.
+
+    Initialized using a directory path."""
+
+    def __init__(self, path):
+        # type: (str) -> None
+        self._path = path
+        self._lock = apt_pkg.FileLock(os.path.join(path, "lock"))
+
+    def __enter__(self):
+        # type: () -> None
+        try:
+            return self._lock.__enter__()
+        except apt_pkg.Error as e:
+            raise LockFailedException(("Failed to lock directory %s: %s") %
+                                       (self._path, e))
+
+    def __exit__(self, typ, value, traceback):
+        # type: (object, object, object) -> None
+        return self._lock.__exit__(typ, value, traceback)
+
+
 class Cache(object):
     """Dictionary-like package cache.
 
@@ -135,6 +162,11 @@ class Cache(object):
             # Call InitSystem so the change to Dir::State::Status is actually
             # recognized (LP: #320665)
             apt_pkg.init_system()
+
+        # Prepare a lock object (context manager for archive lock)
+        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
+        self._archive_lock = _WrappedLock(archive_dir)
+
         self.open(progress)
 
     def fix_broken(self):
@@ -388,8 +420,17 @@ class Cache(object):
                 reqreinst.add(pkg.get_fullname(pretty=True))
         return reqreinst
 
-    def _run_fetcher(self, fetcher):
-        # type: (apt_pkg.Acquire) -> int
+    def _run_fetcher(self, fetcher, allow_unauthenticated):
+        # type: (apt_pkg.Acquire, Optional[bool]) -> int
+        if allow_unauthenticated is None:
+            allow_unauthenticated = apt_pkg.config.find_b("APT::Get::"
+                                        "AllowUnauthenticated", False)
+
+        untrusted = [item for item in fetcher.items if not item.is_trusted]
+        if untrusted and not allow_unauthenticated:
+            raise UntrustedException("Untrusted packages:\n%s" %
+                "\n".join(i.desc_uri for i in untrusted))
+
         # do the actual fetching
         res = fetcher.run()
 
@@ -412,8 +453,12 @@ class Cache(object):
             raise FetchFailedException(err_msg)
         return res
 
-    def _fetch_archives(self, fetcher, pm):
-        # type: (apt_pkg.Acquire, apt_pkg.PackageManager) -> int
+    def _fetch_archives(self,
+                        fetcher,                # type: apt_pkg.Acquire
+                        pm,                     # type: apt_pkg.PackageManager
+                        allow_unauthenticated=None,  # type: Optional[bool]
+                        ):
+        # type: (...) -> int
         """ fetch the needed archives """
         if self._records is None:
             raise CacheClosedException(
@@ -422,22 +467,17 @@ class Cache(object):
         # this may as well throw a SystemError exception
         if not pm.get_archives(fetcher, self._list, self._records):
             return False
+
         # now run the fetcher, throw exception if something fails to be
         # fetched
-        return self._run_fetcher(fetcher)
+        return self._run_fetcher(fetcher, allow_unauthenticated)
 
-    def _get_archive_lock(self, fetcher):
-        # type: (apt_pkg.Acquire) -> None
-        # get lock
-        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
-        try:
-            fetcher.get_lock(archive_dir)
-        except apt_pkg.Error as e:
-            raise LockFailedException(("Failed to lock archive directory %s: "
-                                       " %s") % (archive_dir, e))
-
-    def fetch_archives(self, progress=None, fetcher=None):
-        # type: (AcquireProgress, apt_pkg.Acquire) -> int
+    def fetch_archives(self,
+                       progress=None,        # type: Optional[AcquireProgress]
+                       fetcher=None,         # type: Optional[apt_pkg.Acquire]
+                       allow_unauthenticated=None,  # type: Optional[bool]
+                       ):
+        # type: (...) -> int
         """Fetch the archives for all packages marked for install/upgrade.
 
         You can specify either an :class:`apt.progress.base.AcquireProgress()`
@@ -448,6 +488,10 @@ class Cache(object):
         an exception of type :class:`FetchFailedException` or
         :class:`FetchCancelledException` is raised.
 
+        The keyword-only parameter *allow_unauthenticated* specifies whether
+        to allow unauthenticated downloads. If not specified, it defaults to
+        the configuration option `APT::Get::AllowUnauthenticated`.
+
         .. versionadded:: 0.8.0
         """
         if progress is not None and fetcher is not None:
@@ -457,10 +501,10 @@ class Cache(object):
         if fetcher is None:
             fetcher = apt_pkg.Acquire(progress)
 
-        self._get_archive_lock(fetcher)
-
-        return self._fetch_archives(fetcher,
-                                    apt_pkg.PackageManager(self._depcache))
+        with self._archive_lock:
+            return self._fetch_archives(fetcher,
+                                        apt_pkg.PackageManager(self._depcache),
+                                        allow_unauthenticated)
 
     def is_virtual_package(self, pkgname):
         # type: (str) -> bool
@@ -520,43 +564,41 @@ class Cache(object):
         sources_list -- Update a alternative sources.list than the default.
         Note that the sources.list.d directory is ignored in this case
         """
-        lockfile = apt_pkg.config.find_dir("Dir::State::Lists") + "lock"
-        lock = apt_pkg.get_lock(lockfile)
-
-        if lock < 0:
-            raise LockFailedException("Failed to lock %s" % lockfile)
-
-        if sources_list:
-            old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
-            old_sources_list_d = apt_pkg.config.find("Dir::Etc::sourceparts")
-            old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
-            apt_pkg.config.set("Dir::Etc::sourcelist",
-                               os.path.abspath(sources_list))
-            apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
-            apt_pkg.config.set("APT::List-Cleanup", "0")
-            slist = apt_pkg.SourceList()
-            slist.read_main_list()
-        else:
-            slist = self._list
-
-        try:
-            if fetch_progress is None:
-                fetch_progress = apt.progress.base.AcquireProgress()
-            try:
-                res = self._cache.update(fetch_progress, slist,
-                                         pulse_interval)
-            except SystemError as e:
-                raise FetchFailedException(e)
-            if not res and raise_on_error:
-                raise FetchFailedException()
-            else:
-                return res
-        finally:
-            os.close(lock)
+        with _WrappedLock(apt_pkg.config.find_dir("Dir::State::Lists")):
             if sources_list:
-                apt_pkg.config.set("Dir::Etc::sourcelist", old_sources_list)
-                apt_pkg.config.set("Dir::Etc::sourceparts", old_sources_list_d)
-                apt_pkg.config.set("APT::List-Cleanup", old_cleanup)
+                old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
+                old_sources_list_d = (
+                    apt_pkg.config.find("Dir::Etc::sourceparts"))
+                old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
+                apt_pkg.config.set("Dir::Etc::sourcelist",
+                                   os.path.abspath(sources_list))
+                apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
+                apt_pkg.config.set("APT::List-Cleanup", "0")
+                slist = apt_pkg.SourceList()
+                slist.read_main_list()
+            else:
+                slist = self._list
+
+            try:
+                if fetch_progress is None:
+                    fetch_progress = apt.progress.base.AcquireProgress()
+                try:
+                    res = self._cache.update(fetch_progress, slist,
+                                             pulse_interval)
+                except SystemError as e:
+                    raise FetchFailedException(e)
+                if not res and raise_on_error:
+                    raise FetchFailedException()
+                else:
+                    return res
+            finally:
+                if sources_list:
+                    apt_pkg.config.set("Dir::Etc::sourcelist",
+                                       old_sources_list)
+                    apt_pkg.config.set("Dir::Etc::sourceparts",
+                                       old_sources_list_d)
+                    apt_pkg.config.set("APT::List-Cleanup",
+                                       old_cleanup)
 
     def install_archives(self, pm, install_progress):
         # type: (apt_pkg.PackageManager, InstallProgress) -> int
@@ -592,8 +634,12 @@ class Cache(object):
             install_progress.finish_update()
         return res
 
-    def commit(self, fetch_progress=None, install_progress=None):
-        # type: (AcquireProgress, InstallProgress) -> bool
+    def commit(self,
+               fetch_progress=None,         # type: Optional[AcquireProgress]
+               install_progress=None,       # type: Optional[InstallProgress]
+               allow_unauthenticated=None,  # type: Optional[bool]
+               ):
+        # type: (...) -> bool
         """Apply the marked changes to the cache.
 
         The first parameter, *fetch_progress*, refers to a FetchProgress()
@@ -602,6 +648,10 @@ class Cache(object):
 
         The second parameter, *install_progress*, is a
         apt.progress.InstallProgress() object.
+
+        The keyword-only parameter *allow_unauthenticated* specifies whether
+        to allow unauthenticated downloads. If not specified, it defaults to
+        the configuration option `APT::Get::AllowUnauthenticated`.
         """
         # FIXME:
         # use the new acquire/pkgmanager interface here,
@@ -620,25 +670,26 @@ class Cache(object):
         with apt_pkg.SystemLock():
             pm = apt_pkg.PackageManager(self._depcache)
             fetcher = apt_pkg.Acquire(fetch_progress)
-            self._get_archive_lock(fetcher)
+            with self._archive_lock:
+                while True:
+                    # fetch archives first
+                    res = self._fetch_archives(fetcher, pm,
+                                               allow_unauthenticated)
 
-            while True:
-                # fetch archives first
-                res = self._fetch_archives(fetcher, pm)
-
-                # then install
-                res = self.install_archives(pm, install_progress)
-                if res == pm.RESULT_COMPLETED:
-                    break
-                elif res == pm.RESULT_FAILED:
-                    raise SystemError("installArchives() failed")
-                elif res == pm.RESULT_INCOMPLETE:
-                    pass
-                else:
-                    raise SystemError("internal-error: unknown result code "
-                                      "from InstallArchives: %s" % res)
-                # reload the fetcher for media swaping
-                fetcher.shutdown()
+                    # then install
+                    res = self.install_archives(pm, install_progress)
+                    if res == pm.RESULT_COMPLETED:
+                        break
+                    elif res == pm.RESULT_FAILED:
+                        raise SystemError("installArchives() failed")
+                    elif res == pm.RESULT_INCOMPLETE:
+                        pass
+                    else:
+                        raise SystemError("internal-error: unknown result "
+                                          "code from InstallArchives: %s" %
+                                          res)
+                    # reload the fetcher for media swaping
+                    fetcher.shutdown()
         return (res == pm.RESULT_COMPLETED)
 
     def clear(self):
@@ -976,7 +1027,7 @@ def _test():
     apt_pkg.config.set("Dir::Cache::Archives", "/tmp/pytest")
     pm = apt_pkg.PackageManager(cache._depcache)
     fetcher = apt_pkg.Acquire(apt.progress.text.AcquireProgress())
-    cache._fetch_archives(fetcher, pm)
+    cache._fetch_archives(fetcher, pm, None)
     #sys.exit(1)
 
     print("Testing filtered cache (argument is old cache)")
