@@ -204,13 +204,16 @@ class TestAuthKeys(testcommon.TestCase):
                 WHEEZY_KEYID, "hkp://localhost:19191")
         with self.assertRaises(apt.auth.AptKeyIDTooShortError):
             apt.auth.add_key_from_keyserver(
-                "0101010178F7FE5C3E65D8AF8B48AD624692555", "hkp://localhost:19191")
+                "0101010178F7FE5C3E65D8AF8B48AD624692555",
+                "hkp://localhost:19191")
         with self.assertRaises(apt.auth.AptKeyIDTooShortError):
             apt.auth.add_key_from_keyserver(
-                "0x0101010178F7FE5C3E65D8AF8B48AD624692555", "hkp://localhost:19191")
+                "0x0101010178F7FE5C3E65D8AF8B48AD624692555",
+                "hkp://localhost:19191")
         with self.assertRaises(apt.auth.AptKeyIDTooShortError):
             apt.auth.add_key_from_keyserver(
-                "0101 0101 78F7 FE5C 3E65 D8AF 8B48 AD62 4692 555", "hkp://localhost:19191")
+                "0101 0101 78F7 FE5C 3E65 D8AF 8B48 AD62 4692 555",
+                "hkp://localhost:19191")
 
     def test_add_key_from_server_mitm(self):
         """Verify that the key fingerprint is verified after download"""
@@ -306,5 +309,6 @@ class TestAuthKeys(testcommon.TestCase):
         if self.orig_proxy is not None:
             os.environ['http_proxy'] = self.orig_proxy
 
+
 if __name__ == "__main__":
     unittest.main()
